@@ -7,20 +7,16 @@ from twisted.internet.error import ConnectionDone
 
 from foolscap import broker, referenceable, vocab
 from foolscap.eventual import eventually
-from foolscap.tokens import SIZE_LIMIT, ERROR, NoLocationHintsError, \
-     BananaError, NegotiationError, RemoteNegotiationError, \
-     DuplicateConnection
+from foolscap.tokens import (SIZE_LIMIT, ERROR,
+                             BananaError, NegotiationError,
+                             RemoteNegotiationError, DuplicateConnection)
 from foolscap.ipb import DeadReferenceError
 from foolscap.banana import int2b128
 from foolscap.logging import log
 from foolscap.logging.log import NOISY, OPERATIONAL, WEIRD, UNUSUAL, CURIOUS
-
+from foolscap.util import isSubstring
 from foolscap import crypto
 
-def isSubstring(small, big):
-    assert type(small) is str and type(big) is str
-    return small in big
-
 def best_overlap(my_min, my_max, your_min, your_max, name):
     """Find the highest integer which is in both ranges (inclusive).
     Raise NegotiationError (using 'name' in the error message) if there
@@ -179,19 +175,20 @@ class Negotiation(protocol.Protocol):
             # offer and decision.
             self.negotiationOffer['negotiation-forced'] = "True"
         self.buffer = ""
-        self.options = {}
+        self._test_options = {}
         # to trigger specific race conditions during unit tests, it is useful
         # to allow certain operations to be stalled for a moment.
-        # self.options will contain a key like debug_slow_connectionMade to
-        # indicate that there should be a 1 second delay between the real
-        # connectionMade and the time our self.connectionMade() method is
-        # invoked. To support this, the first time connectionMade() is
-        # invoked, self.debugTimers['connectionMade'] is set to a 1s
-        # DelayedCall, which fires self.debug_fireTimer('connectionMade',
-        # callable, *args). That will set self.debugTimers['connectionMade']
-        # to None, so the condition is not fired again, then invoke the
-        # actual connectionMade method. When the connection is lost, all
-        # remaining timers will be canceled.
+        # self._test_options will contain a key like
+        # debug_slow_connectionMade to indicate that there should be a 1
+        # second delay between the real connectionMade and the time our
+        # self.connectionMade() method is invoked. To support this, the first
+        # time connectionMade() is invoked,
+        # self.debugTimers['connectionMade'] is set to a 1s DelayedCall,
+        # which fires self.debug_fireTimer('connectionMade', callable,
+        # *args). That will set self.debugTimers['connectionMade'] to None,
+        # so the condition is not fired again, then invoke the actual
+        # connectionMade method. When the connection is lost, all remaining
+        # timers will be canceled.
         self.debugTimers = {}
 
         # if anything goes wrong during negotiation (version mismatch,
@@ -223,7 +220,7 @@ class Negotiation(protocol.Protocol):
         self.connector = connector
         self.target = connector.target
         self.targetHost = targetHost
-        self.options = self.tub.options.copy()
+        self._test_options = self.tub._test_options.copy()
         tubID = self.target.getTubID()
         slave_record = self.tub.slave_table.get(tubID, ("none",0))
         assert isinstance(slave_record, tuple), slave_record
@@ -234,7 +231,7 @@ class Negotiation(protocol.Protocol):
         self.log("initServer", listener=repr(listener))
         self.isClient = False
         self.listener = listener
-        self.options = self.listener.options.copy()
+        self._test_options = self.listener._test_options.copy()
         # the broker class is set when we find out which Tub we should use
 
     def parseLines(self, header):
@@ -255,12 +252,12 @@ class Negotiation(protocol.Protocol):
         self.transport.write("\r\n") # end block
 
     def debug_doTimer(self, name, timeout, call, *args):
-        if (self.options.has_key("debug_slow_%s" % name) and
+        if (self._test_options.has_key("debug_slow_%s" % name) and
             not self.debugTimers.has_key(name)):
             self.log("debug_doTimer(%s)" % name)
             t = reactor.callLater(timeout, self.debug_fireTimer, name)
             self.debugTimers[name] = (t, [(call, args)])
-            cb = self.options["debug_slow_%s" % name]
+            cb = self._test_options["debug_slow_%s" % name]
             if cb is not None and cb is not True:
                 cb()
             return True
@@ -335,7 +332,7 @@ class Negotiation(protocol.Protocol):
         # server timeout first
         if self.debug_doTimer("connectionMade", 1, self.connectionMade):
             return
-        timeout = self.options.get('server_timeout', self.SERVER_TIMEOUT)
+        timeout = self._test_options.get('server_timeout', self.SERVER_TIMEOUT)
         if timeout:
             # oldpb clients will hit this case.
             self.negotiationTimer = reactor.callLater(timeout,
@@ -358,7 +355,8 @@ class Negotiation(protocol.Protocol):
 
     def dataReceived(self, chunk):
         self.log("dataReceived(isClient=%s,phase=%s,options=%s): %r"
-                 % (self.isClient, self.receive_phase, self.options, chunk),
+                 % (self.isClient, self.receive_phase, self._test_options,
+                    chunk),
                  level=NOISY)
         if self.receive_phase == ABANDONED:
             return
@@ -444,7 +442,7 @@ class Negotiation(protocol.Protocol):
                 t[0].cancel()
                 self.debugTimers[k] = None
         if self.isClient:
-            l = self.tub.options.get("debug_gatherPhases")
+            l = self.tub._test_options.get("debug_gatherPhases")
             if l is not None:
                 l.append(self.receive_phase)
         if not self.failureReason:
@@ -480,7 +478,7 @@ class Negotiation(protocol.Protocol):
         tub, redirect = self.listener.lookupTubID(targetTubID)
         if tub:
             self.tub = tub # our tub
-            self.options.update(self.tub.options)
+            self._test_options.update(self.tub._test_options)
             self.brokerClass = self.tub.brokerClass
             self.myTubID = tub.tubID
             self.sendPlaintextServerAndStartENCRYPTED()
@@ -1102,20 +1100,19 @@ class Negotiation(protocol.Protocol):
         # know that this connection has succeeded, so they can stop any other
         # connection attempts still in progress.
         if self.isClient:
-            self.connector.negotiationComplete(self.factory)
+            self.connector.negotiationComplete(self)
 
         # finally let our Tub know that they can start using the new Broker.
         # This will wake up anyone who initiated an outbound connection.
-        self.tub.connectionAttached(b)
         self.tub.brokerAttached(theirTubRef, b, self.isClient)
 
     def negotiationFailed(self):
         reason = self.failureReason
         self.stopNegotiationTimer()
         if self.receive_phase != ABANDONED and self.isClient:
-            eventually(self.connector.negotiationFailed, self.factory, reason)
+            eventually(self.connector.negotiationFailed, self, reason)
         self.receive_phase = ABANDONED
-        cb = self.options.get("debug_negotiationFailed_cb")
+        cb = self._test_options.get("debug_negotiationFailed_cb")
         if cb:
             # note that this gets called with a NegotiationError, not a
             # Failure. ACTUALLY: not true, gets a Failure
@@ -1147,271 +1144,3 @@ class Negotiation(protocol.Protocol):
                      level=OPERATIONAL, umid="pm2kjg")
 
 # TODO: make sure code that examines self.receive_phase handles ABANDONED
-
-class TubConnectorClientFactory(protocol.ClientFactory, object):
-    # this is for internal use only. Application code should use
-    # Tub.getReference(url)
-
-    noisy = False
-
-    def __init__(self, tc, host, logparent):
-        self.tc = tc # the TubConnector
-        self.host = host
-        self._logparent = logparent
-
-    def log(self, *args, **kwargs):
-        kwargs['parent'] = self._logparent
-        kwargs['facility'] = "foolscap.negotiation"
-        return log.msg(*args, **kwargs)
-
-    def __repr__(self):
-        # make it clear which remote Tub we're trying to connect to
-        base = object.__repr__(self)
-        at = base.find(" at ")
-        if at == -1:
-            # our annotation isn't really important, so don't fail just
-            # because we guessed the default __repr__ incorrectly
-            return base
-        assert self.tc.tub.tubID
-        origin = self.tc.tub.tubID[:8]
-        assert self.tc.target.getTubID()
-        target = self.tc.target.getTubID()[:8]
-        return base[:at] + " [from %s]" % origin + " [to %s]" % target + base[at:]
-
-    def startedConnecting(self, connector):
-        self.connector = connector
-
-    def startFactory(self):
-        self.log("Starting factory %r" % self)
-        return protocol.ClientFactory.startFactory(self)
-    def stopFactory(self):
-        self.log("Stopping factory %r" % self)
-        return protocol.ClientFactory.stopFactory(self)
-
-    def disconnect(self):
-        self.log("told to disconnect")
-        self.connector.disconnect()
-
-    def buildProtocol(self, addr):
-        nc = self.tc.tub.negotiationClass # this is usually Negotiation
-        proto = nc(self._logparent)
-        proto.initClient(self.tc, self.host)
-        proto.factory = self
-        return proto
-
-    def clientConnectionFailed(self, connector, reason):
-        self.tc.clientConnectionFailed(self, reason)
-
-
-class TubConnector(object):
-    """I am used to make an outbound connection. I am given a target TubID
-    and a list of locationHints, and I try all of them until I establish a
-    Broker connected to the target. I will consider redirections returned
-    along the way. The first hint that yields a connected Broker will stop
-    the search.
-
-    This is a single-use object. The connection attempt begins as soon as my
-    connect() method is called.
-
-    I live until all but one of the TCP connections I initiated have finished
-    closing down. This means that connection establishment attempts in
-    progress are cancelled, and established connections (the ones which did
-    *not* complete negotiation before the winning connection) have called
-    their connectionLost() methods.
-    """
-
-    failureReason = None
-    CONNECTION_TIMEOUT = 60
-    timer = None
-
-    def __init__(self, parent, tubref):
-        self._logparent = log.msg(format="TubConnector created from "
-                                  "%(fromtubid)s to %(totubid)s",
-                                  fromtubid=parent.tubID,
-                                  totubid=tubref.getTubID(),
-                                  level=OPERATIONAL,
-                                  facility="foolscap.connection")
-        self.tub = parent
-        self.target = tubref
-        hints = []
-        # filter out the hints that we can actually use.. there may be
-        # extensions from the future sitting in this list
-        for h in self.target.getLocations():
-            if h[0] == "tcp":
-                (host, port) = h[1:]
-                hints.append( (host, port) )
-        self.remainingLocations = hints
-        # attemptedLocations keeps track of where we've already tried to
-        # connect, so we don't try them twice.
-        self.attemptedLocations = []
-
-        # pendingConnections contains a (PBClientFactory -> Connector) map
-        # for pairs where connectTCP has started, but negotiation has not yet
-        # completed. We keep track of these so we can shut them down when we
-        # stop connecting (either because one of the connections succeeded,
-        # or because someone told us to give up).
-        self.pendingConnections = {}
-
-    def __repr__(self):
-        s = object.__repr__(self)
-        s = s[:-1]
-        s += " from %s to %s>" % (self.tub.tubID, self.target.getTubID())
-        return s
-
-    def log(self, *args, **kwargs):
-        kwargs['parent'] = self._logparent
-        kwargs['facility'] = "foolscap.connection"
-        return log.msg(*args, **kwargs)
-
-    def connect(self):
-        """Begin the connection process. This should only be called once.
-        This will either result in the successful Negotiation object invoking
-        the parent Tub's brokerAttached() method, our us calling the Tub's
-        connectionFailed() method."""
-        self.tub.connectorStarted(self)
-        if not self.remainingLocations:
-            # well, that's going to make it difficult. connectToAll() will
-            # pass through to checkForFailure(), which will notice our lack
-            # of options and deliver this failureReason to the caller.
-            self.failureReason = Failure(NoLocationHintsError())
-        timeout = self.tub.options.get('connect_timeout',
-                                       self.CONNECTION_TIMEOUT)
-        self.timer = reactor.callLater(timeout, self.connectionTimedOut)
-        self.active = True
-        self.connectToAll()
-
-    def stopConnectionTimer(self):
-        if self.timer:
-            self.timer.cancel()
-            del self.timer
-
-    def shutdown(self):
-        self.active = False
-        self.remainingLocations = []
-        self.stopConnectionTimer()
-        for c in self.pendingConnections.values():
-            c.disconnect()
-        # as each disconnect() finishes, it will either trigger our
-        # clientConnectionFailed or our negotiationFailed methods, both of
-        # which will trigger checkForIdle, and the last such message will
-        # invoke self.tub.connectorFinished()
-
-    def connectToAll(self):
-        while self.remainingLocations:
-            location = self.remainingLocations.pop()
-            if location in self.attemptedLocations:
-                continue
-            self.attemptedLocations.append(location)
-            host, port = location
-            lp = self.log("connectTCP to %s" % (location,))
-            f = TubConnectorClientFactory(self, host, lp)
-            c = reactor.connectTCP(host, port, f)
-            self.pendingConnections[f] = c
-            # the tcp.Connector that we get back from reactor.connectTCP will
-            # retain a reference to the transport that it creates, so we can
-            # use it to disconnect the established (but not yet negotiated)
-            # connection
-            if self.tub.options.get("debug_stall_second_connection"):
-                # for unit tests, hold off on making the second connection
-                # for a moment. This allows the first connection to get to a
-                # known state.
-                reactor.callLater(0.1, self.connectToAll)
-                return
-        self.checkForFailure()
-
-    def connectionTimedOut(self):
-        self.timer = None
-        why = "no connection established within client timeout"
-        self.failureReason = Failure(NegotiationError(why))
-        self.shutdown()
-        self.failed()
-
-    def clientConnectionFailed(self, factory, reason):
-        # this is called if some individual TCP connection cannot be
-        # established
-        if not self.failureReason:
-            self.failureReason = reason
-        del self.pendingConnections[factory]
-        self.checkForFailure()
-        self.checkForIdle()
-
-    def redirectReceived(self, newLocation):
-        # the redirected connection will disconnect soon, which will trigger
-        # negotiationFailed(), so we don't have to do a
-        # del self.pendingConnections[factory]
-        self.remainingLocations.append(newLocation)
-        self.connectToAll()
-
-    def negotiationFailed(self, factory, reason):
-        # this is called if protocol negotiation cannot be established, or if
-        # the connection is closed for any reason prior to switching to the
-        # Banana protocol
-        assert isinstance(reason, Failure), \
-               "Hey, %s isn't a Failure" % (reason,)
-        if (not self.failureReason or
-            isinstance(reason, NegotiationError)):
-            # don't let mundane things like ConnectionFailed override the
-            # actually significant ones like NegotiationError
-            self.failureReason = reason
-        del self.pendingConnections[factory]
-        self.checkForFailure()
-        self.checkForIdle()
-
-    def negotiationComplete(self, factory):
-        # 'factory' has just completed negotiation, so abandon all the other
-        # connection attempts
-        self.log("negotiationComplete, %s won" % factory)
-        self.active = False
-        if self.timer:
-            self.timer.cancel()
-            self.timer = None
-        del self.pendingConnections[factory] # this one succeeded
-        for f in self.pendingConnections.keys(): # abandon the others
-            # for connections that are not yet established, this will trigger
-            # clientConnectionFailed. For connections that are established
-            # (and exchanging negotiation messages), this does
-            # loseConnection() and will thus trigger negotiationFailed.
-            f.disconnect()
-        self.checkForIdle()
-
-    def checkForFailure(self):
-        if not self.active:
-            return
-        if self.remainingLocations:
-            return
-        if self.pendingConnections:
-            return
-        # we have no more options, so the connection attempt will fail. The
-        # getBrokerForTubRef may have succeeded, however, if the other side
-        # tried to connect to us at exactly the same time, they were the
-        # master, they established their connection first (but the final
-        # decision is still in flight), and they hung up on our connection
-        # because they felt it was a duplicate. So, if self.failureReason
-        # indicates a duplicate connection, do not signal a failure here. We
-        # leave the connection timer in place in case they lied about having
-        # a duplicate connection ready to go.
-        if (self.failureReason.check(RemoteNegotiationError) and
-            isSubstring(self.failureReason.value.args[0],
-                        "Duplicate connection")):
-            self.log("TubConnector.checkForFailure: connection attempt "
-                     "failed because the other end decided ours was a "
-                     "duplicate connection, so we won't signal the "
-                     "failure here")
-            return
-        self.failed()
-
-    def failed(self):
-        self.stopConnectionTimer()
-        self.active = False
-        self.tub.connectionFailed(self.target, self.failureReason)
-        self.tub.connectorFinished(self)
-
-    def checkForIdle(self):
-        if self.remainingLocations:
-            return
-        if self.pendingConnections:
-            return
-        # we have no more outstanding connections (either in progress or in
-        # negotiation), so this connector is finished.
-        self.log("connectorFinished (%s)" % self)
-        self.tub.connectorFinished(self)
