@@ -6,8 +6,9 @@ from twisted.internet import defer, protocol, error
 from twisted.application import service, internet
 from twisted.python.failure import Failure
 
-from foolscap import ipb, base32, negotiate, broker, observer, eventual, storage
-from foolscap import util
+from foolscap import ipb, base32, negotiate, broker, eventual, storage
+from foolscap import connection, util
+from foolscap.connection_plugins import DefaultTCP
 from foolscap.referenceable import SturdyRef
 from foolscap.tokens import PBError, BananaError, WrongTubIdError, \
      WrongNameError, NoLocationError
@@ -31,19 +32,18 @@ def parse_strport(port):
     # TODO: IPv6
     return (portnum, interface)
 
-Listeners = []
-class Listener(protocol.ServerFactory):
-    """I am responsible for a single listening port, which may connect to
-    multiple Tubs. I have a strports-based Service, which I will attach as a
-    child of one of my Tubs. If that Tub disconnects, I will reparent the
-    Service to a remaining one. """
+class Listener(protocol.ServerFactory, service.MultiService):
+    """I am responsible for a single listening port., which may connect to a
+    single Tub (or none). I have a strports-based Service, which I will
+    attach as a child."""
 
     noisy = False
 
     # this also serves as the ServerFactory
 
-    def __init__(self, port, options={},
-                 negotiationClass=negotiate.Negotiation):
+    def __init__(self, port, _test_options={},
+                 negotiationClass=negotiate.Negotiation,
+                 tub=None):
         """
         @type port: string
         @param port: a L{twisted.application.strports} -style description,
@@ -55,15 +55,15 @@ class Listener(protocol.ServerFactory):
         #  tcp:80:interface=127.0.0.1
         # we reject UNIX sockets.. I don't know if they ever worked.
 
+        service.MultiService.__init__(self)
         portnum, interface = parse_strport(port)
         self.port = port
-        self.options = options
+        self._test_options = _test_options
         self.negotiationClass = negotiationClass
-        self.parentTub = None
-        self.tubs = {}
+        self.tub = tub
         self.redirects = {}
         self.s = internet.TCPServer(portnum, self, interface=interface)
-        Listeners.append(self)
+        self.s.setServiceParent(self)
 
     def getPortnum(self):
         """When this Listener was created with a port string of '0' or
@@ -81,45 +81,12 @@ class Listener(protocol.ServerFactory):
         return self.s._port.getHost().port
 
     def __repr__(self):
-        if self.tubs:
-            return "<Listener at 0x%x on %s with tubs %s>" % (
-                abs(id(self)),
-                self.port,
-                ",".join([str(k) for k in self.tubs.keys()]))
+        if self.tub:
+            return "<Listener at 0x%x on %s with tub %s>" % \
+                   (abs(id(self)), self.port, str(self.tub.tubID))
         return "<Listener at 0x%x on %s with no tubs>" % (abs(id(self)),
                                                           self.port)
 
-    def addTub(self, tub):
-        if tub.tubID in self.tubs:
-            assert tub.tubID is not None
-            raise RuntimeError("This Listener (on %s) is already connected "
-                               "to TubID '%s'" % (self.port, tub.tubID))
-        self.tubs[tub.tubID] = tub
-        if self.parentTub is None:
-            self.parentTub = tub
-            self.s.setServiceParent(self.parentTub)
-
-    def removeTub(self, tub):
-        # this might return a Deferred, since the removal might cause the
-        # Listener to shut down. It might also return None.
-        del self.tubs[tub.tubID]
-        if self.parentTub is tub:
-            # we need to switch to a new one
-            tubs = self.tubs.values()
-            if tubs:
-                self.parentTub = tubs[0]
-                # TODO: I want to do this without first doing
-                # disownServiceParent, so the port remains listening. Can we
-                # do this? It looks like setServiceParent does
-                # disownServiceParent first, so it may glitch.
-                self.s.setServiceParent(self.parentTub)
-            else:
-                # no more tubs, this Listener will go away now
-                d = self.s.disownServiceParent()
-                Listeners.remove(self)
-                return d
-        return None
-
     def getService(self):
         return self.s
 
@@ -149,7 +116,10 @@ class Listener(protocol.ServerFactory):
         return proto
 
     def lookupTubID(self, tubID):
-        return self.tubs.get(tubID), self.redirects.get(tubID)
+        tub = None
+        if tubID == self.tub.tubID:
+            tub = self.tub
+        return (tub, self.redirects.get(tubID))
 
 def generateSwissnumber(bits):
     bytes = os.urandom(bits/8)
@@ -190,10 +160,11 @@ class Tub(service.MultiService):
                      You may provide certData, or certFile, (or neither), but
                      not both.
 
-    @param options: a dictionary of options that can influence connection
-                    connection negotiation. Currently defined keys are:
-                     - debug_slow: if True, wait half a second between
-                                   each negotiation response
+    @param _test_options: a dictionary of options that can influence
+                          connection connection negotiation. Currently
+                          defined keys are:
+                          - debug_slow: if True, wait half a second between
+                                        each negotiation response
 
     @ivar brokers: maps TubIDs to L{Broker} instances
 
@@ -221,9 +192,9 @@ class Tub(service.MultiService):
     disconnectTimeout = None # disconnect after this much idle time
     tubID = None
 
-    def __init__(self, certData=None, certFile=None, options={}):
+    def __init__(self, certData=None, certFile=None, _test_options={}):
         service.MultiService.__init__(self)
-        self.setup(options)
+        self.setup(_test_options)
         if certFile:
             self.setupEncryptionFile(certFile)
         else:
@@ -263,8 +234,8 @@ class Tub(service.MultiService):
     def getIncarnationString(self):
         return self.incarnation_string
 
-    def setup(self, options):
-        self.options = options
+    def setup(self, _test_options):
+        self._test_options = _test_options
         self.logger = flog.theLogger
         self.listeners = []
         self.locationHints = []
@@ -293,13 +264,8 @@ class Tub(service.MultiService):
         self.brokers = {} # maps TubRef to a Broker that connects to them
         self.reconnectors = []
 
-        # all connections, used or not, go in here, so we can stall
-        # stopService() until they're all gone.
-        self._allConnections = set()
-        self._allConnectionsAreDisconnected = observer.OneShotObserverList()
-
+        self._connectionHandlers = {"tcp": DefaultTCP()}
         self._activeConnectors = []
-        self._allConnectorsAreFinished = observer.OneShotObserverList()
 
         self._pending_getReferences = [] # list of (d, furl) pairs
 
@@ -313,6 +279,7 @@ class Tub(service.MultiService):
 
         self._handle_old_duplicate_connections = False
         self._expose_remote_exception_types = True
+        self.accept_gifts = True
 
     def setOption(self, name, value):
         if name == "logLocalFailures":
@@ -359,9 +326,18 @@ class Tub(service.MultiService):
             self._handle_old_duplicate_connections = int(value)
         elif name == "expose-remote-exception-types":
             self._expose_remote_exception_types = bool(value)
+        elif name == "accept-gifts":
+            self.accept_gifts = bool(value)
         else:
             raise KeyError("unknown option name '%s'" % name)
 
+    def removeAllConnectionHintHandlers(self):
+        self._connectionHandlers = {}
+
+    def addConnectionHintHandler(self, hint_type, handler):
+        assert ipb.IConnectionHintHandler.providedBy(handler)
+        self._connectionHandlers[hint_type] = handler
+
     def setLogGathererFURL(self, gatherer_furl_or_furls):
         assert not self._log_gatherer_furls
         if isinstance(gatherer_furl_or_furls, basestring):
@@ -518,50 +494,36 @@ class Tub(service.MultiService):
         d.addCallback(_got_local_ip)
         return d
 
-    def listenOn(self, what, options={}):
+    def listenOn(self, what, _test_options={}):
         """Start listening for connections.
 
-        @type  what: string or Listener instance
-        @param what: a L{twisted.application.strports} -style description,
-                     or a L{Listener} instance returned by a previous call to
-                     listenOn.
-        @param options: a dictionary of options that can influence connection
-                        negotiation before the target Tub has been determined
+        @type  what: string
+        @param what: a L{twisted.application.strports} -style description
+        @param _test_options: a dictionary of options that can influence
+                              connection negotiation before the target Tub
+                              has been determined
 
         @return: The Listener object that was created. This can be used to
-        stop listening later on, to have another Tub listen on the same port,
-        and to figure out which port was allocated when you used a strports
-        specification of 'tcp:0'. """
+        stop listening later on, and to figure out which port was allocated
+        when you used a strports specification of 'tcp:0'."""
 
-        if type(what) is str:
-            l = Listener(what, options, self.negotiationClass)
-        else:
-            assert not options
-            l = what
-        assert l not in self.listeners
-        l.addTub(self)
+        if not isinstance(what, str):
+            raise TypeError("listenOn only accepts str")
+        l = Listener(what, _test_options, self.negotiationClass, tub=self)
         self.listeners.append(l)
+        l.setServiceParent(self)
         return l
 
     def stopListeningOn(self, l):
         # this returns a Deferred when the port is shut down
         self.listeners.remove(l)
-        d = defer.maybeDeferred(l.removeTub, self)
-        return d
+        return l.disownServiceParent()
 
     def getListeners(self):
         """Return the set of Listener objects that allow the outside world to
         connect to this Tub."""
         return self.listeners[:]
 
-    def clone(self):
-        """Return a new Tub (with a different ID), listening on the same
-        ports as this one."""
-        t = Tub()
-        for l in self.listeners:
-            t.listenOn(l)
-        return t
-
     def getTubID(self):
         return self.tubID
     def getShortTubID(self):
@@ -573,8 +535,6 @@ class Tub(service.MultiService):
     def connectorFinished(self, c):
         if c in self._activeConnectors:
             self._activeConnectors.remove(c)
-            if not self.running and not self._activeConnectors:
-                self._allConnectorsAreFinished.fire(self)
 
     def startService(self):
         service.MultiService.startService(self)
@@ -599,31 +559,25 @@ class Tub(service.MultiService):
         self.startService = self._tubsAreNotRestartable
         self.getReference = self._tubHasBeenShutDown
         self.connectTo = self._tubHasBeenShutDown
+
+        # Tell everything to shut down now. We assume that it will stop
+        # twitching by the next tick, so Trial unit tests won't complain
+        # about a dirty reactor. We wait on a few things that might not
+        # behave.
         dl = []
         for rc in list(self.reconnectors):
             rc.stopConnecting()
         del self.reconnectors
-        for l in list(self.listeners):
-            # TODO: rethink this, what I want is for stopService to cause all
-            # Listeners to shut down, but I'm not sure this is the right way
-            # to do it.
-            d = l.removeTub(self)
-            if isinstance(d, defer.Deferred):
-                dl.append(d)
-        dl.append(service.MultiService.stopService(self))
-
-        if self._activeConnectors:
-            dl.append(self._allConnectorsAreFinished.whenFired())
         for c in list(self._activeConnectors):
             c.shutdown()
-
-        if self._allConnections:
-            dl.append(self._allConnectionsAreDisconnected.whenFired())
         why = Failure(error.ConnectionDone("Tub.stopService was called"))
         for b in self.brokers.values():
             b.shutdown(why, fireDisconnectWatchers=False)
 
-        return defer.DeferredList(dl)
+        d = defer.DeferredList(dl)
+        d.addCallback(lambda _: service.MultiService.stopService(self))
+        d.addCallback(eventual.fireEventually)
+        return d
 
     def generateSwissnumber(self, bits):
         return generateSwissnumber(bits)
@@ -658,8 +612,8 @@ class Tub(service.MultiService):
         """
 
         if not self.locationHints:
-            raise RuntimeError("you must setLocation() before "
-                               "you can registerReference()")
+            raise NoLocationError("you must setLocation() before "
+                                  "you can registerReference()")
         oldfurl = None
         if furlFile:
             try:
@@ -921,7 +875,7 @@ class Tub(service.MultiService):
         if tubref not in self.tubConnectors:
             # the TubConnector will call our brokerAttached when it finishes
             # negotiation, which will fire waitingForBrokers[tubref].
-            c = negotiate.TubConnector(self, tubref)
+            c = connection.TubConnector(self, tubref, self._connectionHandlers)
             self.tubConnectors[tubref] = c
             c.connect()
 
@@ -941,8 +895,6 @@ class Tub(service.MultiService):
         b2.setTub(self)
         t1.protocol = b1; t2.protocol = b2
         b1.makeConnection(t1); b2.makeConnection(t2)
-        self.connectionAttached(b1)
-        self.connectionAttached(b2)
         self.brokerAttached(tubref, b1, False)
         return b1
 
@@ -966,9 +918,6 @@ class Tub(service.MultiService):
             for d in waiting:
                 d.errback(why)
 
-    def connectionAttached(self, broker):
-        self._allConnections.add(broker)
-
     def brokerAttached(self, tubref, broker, isClient):
         assert self.running
         assert tubref
@@ -1007,13 +956,6 @@ class Tub(service.MultiService):
             if self.brokers[tubref] is broker:
                 del self.brokers[tubref]
 
-    def connectionDropped(self, broker):
-        self._allConnections.discard(broker)
-        # if the Tub has already shut down, we may need to notify observers
-        # who are waiting for all of our connections to finish shutting down.
-        if (not self.running and not self._allConnections):
-            self._allConnectionsAreDisconnected.fire(self)
-
     def debug_listBrokers(self):
         # return a list of (tubref, inbound, outbound) tuples. The tubref
         # tells you which broker this is, 'inbound' is a list of
