@@ -437,7 +437,7 @@ class TestCallable(unittest.TestCase):
             self.failUnlessEqual(rref.getRemoteTubID(), self.tubB.getTubID())
             self.failUnlessEqual(rref.isConnected(), True)
             self.failUnlessEqual(rref.getLocationHints(),
-                                 [('tcp', '127.0.0.1', self.tub_ports[1])])
+                                 ['127.0.0.1:%d' % self.tub_ports[1]])
         d.addCallback(_check)
         return d
 
