@@ -11,7 +11,8 @@ from foolscap.referenceable import RemoteReference
 from foolscap.eventual import eventually, fireEventually, flushEventualQueue
 from foolscap.test.common import HelperTarget, TargetMixin, ShouldFailMixin, \
      StallMixin
-from foolscap.tokens import WrongTubIdError, PBError, NoLocationHintsError
+from foolscap.tokens import WrongTubIdError, PBError, NoLocationHintsError, \
+    NoLocationError
 
 # create this data with:
 #  t = Tub()
@@ -93,6 +94,14 @@ class SetLocation(unittest.TestCase):
         # setLocation may only be called once
         self.failUnlessRaises(PBError, t.setLocation, "127.0.0.1:12345")
 
+    def test_unreachable(self):
+        t = Tub()
+        t.setServiceParent(self.s)
+        # we call neither .listenOn nor .setLocation
+        self.failUnlessEqual(t.locationHints, [])
+        self.failUnlessRaises(NoLocationError,
+                              t.registerReference, Referenceable())
+
     def test_set_location_automatically(self):
         t = Tub()
         l = t.listenOn("tcp:0")
@@ -103,8 +112,8 @@ class SetLocation(unittest.TestCase):
             sr = SturdyRef(furl)
             portnum = l.getPortnum()
             for lh in sr.locationHints:
-                self.failUnlessEqual(lh[2], portnum, lh)
-            self.failUnless(("tcp", "127.0.0.1", portnum) in sr.locationHints)
+                self.failUnlessEqual(lh.split(":")[1], str(portnum), lh)
+            self.failUnless("127.0.0.1:%d" % portnum in sr.locationHints)
         d.addCallback(_check)
         return d
 
@@ -424,7 +433,9 @@ class BadLocationFURL(unittest.TestCase):
         tubB = Tub()
         tubB.setServiceParent(self.s)
 
-        tubB.setLocation("") # this is how you say "unrouteable"
+        # This is a hack to get a FURL with empty location hints. The correct
+        # way to make a Tub unreachable is to not call .setLocation() at all.
+        tubB.setLocation("")
         r = Receiver(tubB)
         furl = tubB.registerReference(r)
         # the buggy behavior is that the following call raises an exception
@@ -438,9 +449,6 @@ class BadLocationFURL(unittest.TestCase):
         return d
 
     def test_future(self):
-        # bug #129: a FURL with no location hints causes a synchronous
-        # exception in Tub.getReference(), instead of an errback'ed Deferred.
-
         tubA = Tub()
         tubA.setServiceParent(self.s)
         tubB = Tub()
